--
-- DMVs
--

select * from sys.dm_exec_query_stats
select * from sys.dm_exec_sql_text(0x03000400951C390E9E4C0D00F8A5000001000000000000000000000000000000000000000000000000000000)
select * from sys.dm_exec_query_plan(0x05000400951C390EC0FE56A0B600000001000000000000000000000000000000000000000000000000000000) 

--
-- Find slowest queries
--

SELECT TOP 10
			CAST(qs.total_elapsed_time / 1000000.0 AS DECIMAL(28, 2)) AS [Total Elapsed Duration (s)],
			qs.execution_count,
			SUBSTRING (qt.text,(qs.statement_start_offset/2) + 1,
			((CASE WHEN qs.statement_end_offset = -1
				THEN LEN(CONVERT(NVARCHAR(MAX), qt.text)) * 2
				ELSE
					qs.statement_end_offset
				END - qs.statement_start_offset)/2) + 1) AS [Individual Query],
			qt.text AS [Parent Query],
			DB_NAME(qt.dbid) AS DatabaseName,
			qp.query_plan
FROM		sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) qt
CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle) qp
INNER JOIN	sys.dm_exec_cached_plans cp	ON qs.plan_handle=cp.plan_handle
ORDER BY	total_elapsed_time DESC;

--
-- Missing indexes
--

SELECT		
			DB_NAME(mid.database_id) as DatabaseName,  
			CONVERT (decimal (28,1), 
			migs.avg_total_user_cost * migs.avg_user_impact * (migs.user_seeks + migs.user_scans)
			) AS improvement_measure, 
			'CREATE INDEX missing_index_' + CONVERT (nvarchar, mig.index_group_handle) + '_' + CONVERT (nvarchar, mid.index_handle) 
			+ ' ON ' + mid.statement 
			+ ' (' + ISNULL (mid.equality_columns,'') 
			+ CASE WHEN mid.equality_columns IS NOT NULL AND mid.inequality_columns IS NOT NULL THEN ',' ELSE '' END 
			+ ISNULL (mid.inequality_columns, '')
			+ ')' 
			+ ISNULL (' INCLUDE (' + mid.included_columns + ')', '') AS create_index_statement, 
			mig.index_group_handle,
			mid.index_handle,
			migs.[group_handle],
			migs.[unique_compiles],
			migs.[user_seeks],
			migs.[user_scans],
			migs.[last_user_seek],
			migs.[last_user_scan],
			migs.[avg_total_user_cost],
			migs.[avg_user_impact],
			migs.[system_seeks],
			migs.[system_scans],
			migs.[last_system_seek],
			migs.[last_system_scan],
			migs.[avg_total_system_cost],
			migs.[avg_system_impact]	,		
			mid.[object_id],
			statement           
FROM		sys.dm_db_missing_index_groups mig
INNER JOIN	sys.dm_db_missing_index_group_stats migs ON migs.group_handle = mig.index_group_handle
INNER JOIN	sys.dm_db_missing_index_details mid ON mig.index_handle = mid.index_handle
WHERE		CONVERT (decimal (28,1), 
			migs.avg_total_user_cost * migs.avg_user_impact * (migs.user_seeks + migs.user_scans)) > 10
ORDER BY	migs.avg_total_user_cost * migs.avg_user_impact * (migs.user_seeks + migs.user_scans) DESC;



