--
-- Indici errati
--

SET NOCOUNT ON

--Change database connection
USE AdventureWorks2014;

--Create tables with test data
IF OBJECT_ID('tTable_data_1') IS NOT NULL
DROP TABLE tTable_data_1;

CREATE TABLE tTable_data_1
(
Col1 VARCHAR(10) 
,Col2 VARCHAR(10) 
);

IF OBJECT_ID('tTable_data_2') IS NOT NULL
DROP TABLE tTable_data_2;

CREATE TABLE tTable_data_2
(
Col1 VARCHAR(10) 
,Col2 VARCHAR(10) 
);

--Populate table with data
--This method will cause page splits in the clustered index and leave the index fragmented
DECLARE @intCounter INT
SET @intCounter = 1

WHILE @intCounter < 10001
BEGIN

	INSERT INTO tTable_data_1 VALUES(@intCounter,@intCounter)
	INSERT INTO tTable_data_2 VALUES(@intCounter,@intCounter)
 	
	SET @intCounter = @intCounter + 1
END;

--Execute query against tables
SET STATISTICS PROFILE ON;
SET STATISTICS IO ON;

--Table scans performed
SELECT t1.col2, t2.col2
FROM tTable_data_1 t1
INNER JOIN tTable_data_2 t2
ON t1.col1 = t2.col1
WHERE t1.col1 = '2000';
--WHERE t1.col1 = 2000;

--Create index
CREATE NONCLUSTERED INDEX ncl_tTable_data_1 ON tTable_data_1(Col1);
CREATE NONCLUSTERED INDEX ncl_tTable_data_2 ON tTable_data_2(Col1);

--Execute query against tables
--Bookmark lookups performed
SELECT t1.col2, t2.col2
FROM tTable_data_1 t1
INNER JOIN tTable_data_2 t2
ON t1.col1 = t2.col1
WHERE t1.col1 = '2000';
--WHERE t1.col1 = 2000;

--Correct indexes
DROP INDEX tTable_data_1.ncl_tTable_data_1;
DROP INDEX tTable_data_2.ncl_tTable_data_2;

CREATE NONCLUSTERED INDEX ncl_tTable_data_1 ON tTable_data_1(Col1,Col2);
CREATE NONCLUSTERED INDEX ncl_tTable_data_2 ON tTable_data_2(Col1,Col2);

--No table scans or bookmark lookups
SELECT t1.col2, t2.col2
FROM tTable_data_1 t1
INNER JOIN tTable_data_2 t2
ON t1.col1 = t2.col1
WHERE t1.col1 = '2000';
--WHERE t1.col1 = 2000;

SET STATISTICS PROFILE OFF;
SET STATISTICS IO OFF;

--Drop tables
IF OBJECT_ID('tTable_data_1') IS NOT NULL
DROP TABLE tTable_data_1;

IF OBJECT_ID('tTable_data_2') IS NOT NULL
DROP TABLE tTable_data_2;


